﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UF.AssessmentProject.Helper;

namespace UF.AssessmentProject.Providers
{
    public class HomeProvider : baseProvider
    {
     
        public Model.Transaction.ResponseMessage validateField(Model.Transaction.RequestMessage req)
        {
            var resultmess = "";
            if (req.partnerkey == null)
                resultmess += "partnerkey is required!, ";
            if (req.partnerrefno == null)
                resultmess += "partnerrefno is required!, ";
            if (req.timestamp == null)
                resultmess += "timestamp is required!, ";
            if (req.sig == null)
                resultmess += "sig is required!, ";
            if(resultmess.Length > 0)
                return new Model.Transaction.ResponseMessage
                {
                    success = DataDictionary.responseResult.failed,
                    resultmessage = resultmess.Substring(0,resultmess.Length-2),
                };
            return new Model.Transaction.ResponseMessage
            {
                success = DataDictionary.responseResult.success,
            };
        }


        public Model.Transaction.ResponseMessage Validate(Model.Transaction.RequestMessage req)
        {

            var rs = validateField(req);
            if (rs.success == DataDictionary.responseResult.failed)
                return rs;


            if (DateTime.Now >= DateTime.Parse(req.timestamp).AddMinutes(5))
                return new Model.Transaction.ResponseMessage
                {
                    success = DataDictionary.responseResult.failed,
                    resultmessage = "Expired!"
                };


            var rawsig = DateTime.Parse(req.timestamp).ToString("yyyyMMddHHmmss") + req.partnerkey + req.partnerrefno + req.totalamount + req.partnerpassword;
            string sig = CommonUtils.EncodeBase64((CommonUtils.ComputeSha256Hash(rawsig)));
            var partner = db.Partners.Where(c => c.partnerkey == req.partnerkey).FirstOrDefault();
            string encodepass = CommonUtils.EncodeBase64(partner.partnerpassword);

            if (partner == null || (partner != null && encodepass != req.partnerpassword) || sig != req.sig)
                return new Model.Transaction.ResponseMessage
                {
                    success = DataDictionary.responseResult.failed,
                    resultmessage = "Access Denied!",
                };


            long total = 0;
            foreach (UF.AssessmentProject.Model.Transaction.itemdetail item in req.items)
            {
                if( item.name.Length < 0)
                    return new Model.Transaction.ResponseMessage
                    {
                        success = DataDictionary.responseResult.failed,
                        resultmessage = "item name cannot be null!",
                    };

                if ( item.partneritemref.Length < 0)
                    return new Model.Transaction.ResponseMessage
                    {
                        success = DataDictionary.responseResult.failed,
                        resultmessage = "partneritemref cannot be null!",
                    };

                if( item.unitprice < 0 )
                    return new Model.Transaction.ResponseMessage
                    {
                        success = DataDictionary.responseResult.failed,
                        resultmessage = "unitprice only allow positive value!",
                    };

                if ( item.qty >0 && item.qty <=5)
                    total += item.qty * item.unitprice;
                else
                    return new Model.Transaction.ResponseMessage
                    {
                        success = DataDictionary.responseResult.failed,
                        resultmessage = "qty only allow positive and not exceed 5",
                    };

            }

            req.totalamount = total;

            return new Model.Transaction.ResponseMessage
            {
                success = DataDictionary.responseResult.success,
                resultmessage = "Request data is valid.",
            };
        }

    }
}
